'Reprsentiert einen Datentrger
Public Class DataMedium
    'Die maximale Gre des Datentrgers
    Public Const c_MaxSize As Long = 700 '700 MB
    Public ReadOnly Property MaxSize() As Long
        Get
            Return c_MaxSize
        End Get
    End Property

    'Die aktuelle Restgre des Datentrgers
    Private m_SizeLeft As Long
    Public ReadOnly Property SizeLeft() As Long
        Get
            Return m_SizeLeft
        End Get
    End Property

    'Die Liste der enthaltenen Dateien
    Private m_Files As List(Of DataMediumFile)
    Public ReadOnly Property Files() As List(Of DataMediumFile)
        Get
            Return m_Files
        End Get
    End Property

    'Fgt der Liste eine Datei hinzu, sofern die Gre ausreicht
    Public Sub AddFile(ByVal File As DataMediumFile)
        If m_SizeLeft - File.Size >= 0 Then
            m_Files.Add(File)
            m_SizeLeft -= File.Size
        End If
    End Sub

    'Entfernt eine Datei aus der Liste
    Public Sub RemoveFile(ByVal File As DataMediumFile)
        If m_Files.Contains(File) Then
            m_Files.Remove(File)
            m_SizeLeft += File.Size
        End If
    End Sub

    Public Sub New()
        m_Files = New List(Of DataMediumFile)
        m_SizeLeft = c_MaxSize
    End Sub

    Public Sub New(ByVal nFiles As IEnumerable(Of DataMediumFile))
        Me.New()
        For Each file As DataMediumFile In nFiles
            AddFile(file)
        Next
    End Sub

    'Reprsentiert eine Datei auf einem Datentrger
    Public Class DataMediumFile
        Implements IComparable(Of DataMediumFile)

        'Der Name der Datei
        Private m_Name As String
        Public Property Name() As String
            Get
                Return m_Name
            End Get
            Set(ByVal value As String)
                If ValidName(value) Then
                    m_Name = value
                Else
                    Throw New NotSupportedException
                End If
            End Set
        End Property

        'Die Gre der Datei
        Private m_Size As Long
        Public Property Size() As Long
            Get
                Return m_Size
            End Get
            Set(ByVal value As Long)
                If ValidSize(value) Then
                    m_Size = value
                Else
                    Throw New NotSupportedException
                End If
            End Set
        End Property

        'Gibt sinnvollen Text aus (wird fr Ausgabe in Listbox bentigt)
        Public Overrides Function ToString() As String
            Return m_Name & ": " & m_Size.ToString("D")
        End Function

        'Damit die Dateien nach der Gre sortiert werden
        Public Function CompareTo(ByVal other As DataMediumFile) As Integer Implements System.IComparable(Of DataMediumFile).CompareTo
            Return Size.CompareTo(other.Size)
        End Function

        Public Sub New(ByVal nName As String, ByVal nSize As Long)
            If ValidName(nName) And ValidSize(nSize) Then
                m_Name = nName
                m_Size = nSize
            Else
                Throw New NotSupportedException
            End If
        End Sub

        Public Shared Function ValidName(ByVal nName As String) As Boolean
            If nName <> "" Then
                Return True
            Else
                Return False
            End If
        End Function

        Public Shared Function ValidSize(ByVal nSize As Long) As Boolean
            If nSize > 0 And nSize < c_MaxSize Then
                Return True
            Else
                Return False
            End If
        End Function
    End Class
End Class